/*---------------------------------------------------------------------
  Predicate: cpt(+ChildInst,+ParentsInst,-Prob)
       Args: 
   Succeeds: exactly once
   Comments: Generic CPTs - markers are part of the instantiation
             All instantiations are assumed complete
---------------------------------------------------------------------*/
cpt(_Child-(Marker-alleles(A1,A2)),ParentVals,Prob) :-
	A1 @=< A2,
	ordered_alleles(Marker,alleles(A1,A2),ParentVals,Prob1), %A1 from mum, A2 from dad
	ordered_alleles(Marker,alleles(A2,A1),ParentVals,Prob2), %A2 from mum, A1 from dad
	Prob is Prob1 + Prob2.

/*---------------------------------------------------------------------
  Predicate: ordered_alleles(+Marker,+OrderedChildAlleles,+ParentsAlleles,-Prob)
       Args: 
   Succeeds: exactly once
   Comments: Get prob that allele i is from parent i, for i=1,2
---------------------------------------------------------------------*/
ordered_alleles(Marker,alleles(A1,A2),ParentVals,Prob) :- %A1 is maternal gene, A2 paternal
	parent1_alleles(ParentVals,Parent1Alleles),
	parent2_alleles(ParentVals,Parent2Alleles),
	inherit_allele_from_parent_prob(Marker,A1,Parent1Alleles,MotherProb),
	inherit_allele_from_parent_prob(Marker,A2,Parent2Alleles,FatherProb),
	Prob is MotherProb*FatherProb.                         %both events happened, so *
	
/*---------------------------------------------------------------------
  Predicate: parent1_alleles(+ParentsAlleles,-Parent1Alleles)
       Args: 
   Succeeds: exactly once
   Comments: get alleles for first parent, or return unknown if no parents
---------------------------------------------------------------------*/
parent1_alleles([_Parent-(_Marker-Alleles)|_],Alleles) :- !.
parent1_alleles(_,unknown).

/*---------------------------------------------------------------------
  Predicate: parent2_alleles(+ParentsAlleles,-Parent2Alleles)
       Args: 
   Succeeds: exactly once
   Comments: get alleles for second parent, or return unknown if no 2nd parent
---------------------------------------------------------------------*/
parent2_alleles([_,_Parent-(_Marker-Alleles)],Alleles) :- !.
parent2_alleles(_,unknown).

/*---------------------------------------------------------------------
  Predicate: inherit_allele_from_parent_prob(+Marker,+Allele,+ParentalAlleles,-Prob)
       Args: 
   Succeeds: exactly once
   Comments: Prob of inheriting Allele from a parent with ParentalAlleles
---------------------------------------------------------------------*/
inherit_allele_from_parent_prob(Marker,A,unknown,Prob) :-  % don't have parental alleles
	!,             %green cut
	allele_system(Marker,AlleleFreqs,_K,_N),           % just return background frequency
	memberchk(A-Prob,AlleleFreqs).
inherit_allele_from_parent_prob(Marker,A,alleles(PA1,PA2),Prob) :-
	inherit_with_mutation(Marker,PA1,A,P1),
	inherit_with_mutation(Marker,PA2,A,P2),
	Prob is (0.5*P1) + (0.5*P2).                % Section 2.3

/*---------------------------------------------------------------------
  Predicate: inherit_with_mutation(+Marker,+ParentalAllele,+ChildAllele,-Prob)
       Args: 
   Succeeds: exactly once
   Comments: Prob of a child inheriting ChildAllele, given that ParentalAllele was passed down
             ParentAllele might mutate.
---------------------------------------------------------------------*/
%A=B
%o has to be treated differently, since it's actually a set of alleles
inherit_with_mutation(Marker,B,B,P) :-
	!,
	mutation_rate(M),
	(
	  B = o
	->
	  allele_system(Marker,_AlleleFreqs,K,N),
	  P is 1 - (M * (K/(N-1)))        % 3rd line
	;
	  P is 1-M                        % 1st line
	).



%A!=B
inherit_with_mutation(Marker,_A,B,P) :-
	mutation_rate(M),
	allele_system(Marker,_AlleleFreqs,K,N),
	(
	  B = o
	->
	  P is (M/(N-1)) * (N-K)          % 4th line
	;
	  P is M/(N-1)                    % 2nd line                        
	).

possible_vals(Marker,Vals) :-
	allele_system(Marker,AlleleFreqs,_,_),
	findall(
		Marker-alleles(A1,A2),
		(
		  member(A1-_,AlleleFreqs),
		  member(A2-_,AlleleFreqs),
		  A1 @=< A2
		),
		Vals
	       ).